@@ -9,6 +9,8 @@ from   .checktetrahedron import CheckTetrahedron
 
 from   ..utils.protein_sel import three2one
 from   ..utils.rock        import Rock
+from   ..utils.rockchain   import RockChain
+from   ..utils.rockresidue import RockResidue
 from   ..utils.openmmutils import *
 
 import numpy  as np
@@ -26,9 +28,8 @@ class REM:
     # mergedPositions = modeller.positions
 
     def __init__(self, structure=None, out=None, protein=None, refposre=None, outrem=None,
-        rock=None, rockrcut=1.2, rockKbond=5000.0, rockresname='ROCK', rockout='ROCK_rem.pdb',
-        nonrockout='NONROCK.dms',
-        rocktype='HL',
+        rock=None, rockout='ROCK_rem.pdb', nonrockout='NONROCK.dms',
+        rockCtype='CTL3', rockHtype='HAL3', rockprefix='ROCK',
         rcut=1.2, pbc=True, 
         A=100, C=50,
         mapping = [], mapping_add = [], 
@@ -38,12 +39,19 @@ class REM:
         bfactor_posre = 0.5, add_bonds=True, sort=False, version='v4',
         cospower=2, turn_off_torsion_warning=False,
         nsteps=10000,
-        turn_off_EMNVT=False):
+        turn_off_EMNVT=False,
+        T=310):
         
         # v3 should not be used
         # protein: version = 'v4' seems the best
         # lipid:   version = 'v1' seems the best
 
+        self.fcx = fcx
+        self.fcy = fcy
+        self.fcz = fcz
+        self.bfactor_posre = bfactor_posre
+        self.T = T
+
 
         ### NonbondedMethod
         if pbc:
@@ -59,6 +67,7 @@ class REM:
         self.cospower = cospower
         self.nsteps   = nsteps
         u          = Universe(structure)
+        self.protein = protein
 
 
         if not isinstance(ff_add, list):
@@ -67,14 +76,27 @@ class REM:
 
         ### Make a rock file
         if rock:
-            rr         = Rock(structure=rock, out='ROCK', 
-                              rcut=rockrcut, 
-                              Kbond=rockKbond, 
-                              resname=rockresname,
-                              type=rocktype)
+            #rr         = Rock(structure=rock, out='ROCK', 
+            #                  rcut=rockrcut, 
+            #                  Kbond=rockKbond, 
+            #                  resname=rockresname,
+            #                  rockCtype=rockCtype,
+            #                  rockHtype=rockHtype,
+            #                  ENM=rockENM)
+
+            #rr         = RockChain(structure=rock, 
+            #                       out='ROCK', 
+            #                       resname=rockresname,
+            #                       rockCtype=rockCtype,
+            #                       rockHtype=rockHtype)
+
+            rr         = RockResidue(structure=rock, 
+                                     out=rockprefix, 
+                                     rockCtype=rockCtype,
+                                     rockHtype=rockHtype)
 
             rrdms      = DesmondDMSFile(rr.dms)
-            ff_add    += ['ROCK.xml']
+            ff_add    += [rockprefix + '.xml']
 
 
         ### Read XML
@@ -103,20 +125,35 @@ class REM:
                 for bond in bonds:
                     pdb.topology.addBond(pdbatoms[bond[0]], pdbatoms[bond[1]])
 
+            realpbc = pdb.topology.getPeriodicBoxVectors()
+            #print(realpbc)
+
         else:
             raise IOError('Please provide a pdb or dms file')
 
+        fakepbc = Quantity(value=(Vec3(x=9.0, y=0.0, z=0.0), 
+                                  Vec3(x=0.0, y=9.0, z=0.0), 
+                                  Vec3(x=0.0, y=0.0, z=9.0)), 
+                           unit=nanometer)
 
         ### Combine systems (rock should be the first because of the bonds added later)
         modeller_combined = []
         universe_combined = []
 
         if rock:
+            if pbc:
+                rrdms.topology.setPeriodicBoxVectors(realpbc)
+            else:
+                rrdms.topology.setPeriodicBoxVectors(fakepbc)
             modeller_combined.append([rrdms.topology, rrdms.positions])
             universe_combined.append(Universe(rr.dms).atoms)
 
         if protein:
             proteinpdb = PDBFile(protein)
+            if pbc:
+                proteinpdb.topology.setPeriodicBoxVectors(realpbc)
+            else:
+                proteinpdb.topology.setPeriodicBoxVectors(fakepbc)
             modeller_combined.append([proteinpdb.topology, proteinpdb.positions])
             universe_combined.append(Universe(protein).atoms)
         
@@ -131,10 +168,11 @@ class REM:
             modeller.add(modeller_combined[i][0], modeller_combined[i][1])
 
         if pbc:
-            modeller.topology.setPeriodicBoxVectors(pdb.topology.getPeriodicBoxVectors())
+            modeller.topology.setPeriodicBoxVectors(realpbc)
 
         self.final = modeller
         print(self.final.topology)
+        #print(self.final.topology.getPeriodicBoxVectors())
 
 
         ### Make a universe
@@ -167,10 +205,12 @@ class REM:
 
         ### Add posre
         if refposre:
-            self.system.addForce(addRefPosre(u, refposre, fcx, fcy, fcz))
+            # self.system.addForce(addRefPosre(u, refposre, fcx, fcy, fcz))
+            self.system.addForce(addRefPosrePeriodic(u, refposre, fcz))
         else:
             # rock molecules have bfactor of 0.0 -> will have no restraints
-            self.system.addForce(addPosre(u, bfactor_posre, fcx, fcy, fcz))
+            # self.system.addForce(addPosre(u, bfactor_posre, fcx, fcy, fcz))
+            self.system.addForce(addPosrePeriodic(u, bfactor_posre, fcz))
 
 
 
@@ -196,9 +236,9 @@ class REM:
  
         print("Adding Isomer Torsions - started")
         self.system.addForce(addPeptideTorsions(  u, Kpeptide))
-        self.system.addForce(addCisTransTorsions( u, Kcistrans, mapping, exclude=[rockresname], turn_off_torsion_warning=turn_off_torsion_warning))
-        self.system.addForce(addChiralTorsions(   u, Kchiral,   mapping, exclude=[rockresname], turn_off_torsion_warning=turn_off_torsion_warning))
-        self.system.addForce(addDihedralTorsions( u, Kdihedral, mapping, exclude=[rockresname], turn_off_torsion_warning=turn_off_torsion_warning))
+        self.system.addForce(addCisTransTorsions( u, Kcistrans, mapping, turn_off_torsion_warning=turn_off_torsion_warning))
+        self.system.addForce(addChiralTorsions(   u, Kchiral,   mapping, turn_off_torsion_warning=turn_off_torsion_warning))
+        self.system.addForce(addDihedralTorsions( u, Kdihedral, mapping, turn_off_torsion_warning=turn_off_torsion_warning))
         print("Adding Isomer Torsions - finished")
 
 
@@ -211,12 +251,12 @@ class REM:
         print("Running REM without isomeric torsions")
         self.removeForces(['PeptideTorsion', 'CisTransTorsion', 'ChiralTorsion'])
         self.runREM()
+        u.atoms[['x','y','z']] = self.numpypositions
 
         ### Save outREM
-        if outrem:
-            u.atoms[['x','y','z']] = self.numpypositions
-            u.write(outrem)
-            #CheckTetrahedron(outrem, ff=ff, ff_add=ff_add)
+        if outrem: u.write(outrem)
+        #CheckTetrahedron(outrem, ff=ff, ff_add=ff_add)
+        self.u = u
 
         ### Run EM + NVT
         if turn_off_EMNVT:
@@ -231,23 +271,26 @@ class REM:
         u.atoms[['x','y','z']] = self.numpypositions
 
         if rock:
-            rockstruct = Universe(data = u.atoms[u.atoms.resname == 'ROCK'])
+            rockbA = u.atoms.resname.str.startswith('ROCK')
+            rockstruct = Universe(data = u.atoms[rockbA])
             rockstruct.dimensions = u.dimensions
             rockstruct.cell       = u.cell 
-            rockstruct.bodns      = addBonds(rockstruct, xml)
+            #rockstruct.bonds      = addBonds(rockstruct, xml)
+            #rockstruct.bonds      = getBonds(rockstruct, ff=ff, ff_add=ff_add)
             rockstruct.write(rockout)
 
-            nonrockstruct = Universe(data = u.atoms[u.atoms.resname != 'ROCK'])
+            nonrockstruct = Universe(data = u.atoms[~rockbA])
             nonrockstruct.dimensions = u.dimensions
             nonrockstruct.cell       = u.cell
             nonrockstruct.bonds      = addBonds(nonrockstruct, xml)
+            #nonrockstruct.bonds      = getBonds(nonrockstruct, xml)
             nonrockstruct.write(nonrockout)
 
             new = Universe(data = pd.concat([Universe(rock).atoms, nonrockstruct.atoms], ignore_index=True))
             new.dimensions = u.dimensions
             new.cell       = u.cell
             if out: new.write(out)
-            ff_add.remove('ROCK.xml')
+            ff_add.remove(rockprefix + '.xml')
 
         else:
             #if sort: u.sort()
@@ -257,10 +300,14 @@ class REM:
                 #u.bonds = self.bonds
 
             if out:
-                u.write(out, wrap=True)
+                if pbc:
+                    u.write(out, wrap=True)
+                else:
+                    u.write(out)
         
         #CheckTetrahedron(out, ff=ff, ff_add=ff_add)
         self.universe = u
+        self.u        = u
         self.forces   = { force.__class__.__name__ : force for force in self.system.getForces() }
         #print(self.forces.keys())
 
@@ -404,7 +451,7 @@ class REM:
 
 
     def runREM(self):
-        integrator = LangevinIntegrator(310*kelvin, 1/picosecond, 0.002*picoseconds)
+        integrator = LangevinIntegrator(self.T*kelvin, 1/picosecond, 0.002*picoseconds)
         simulation = Simulation(self.final.topology, self.system, integrator)
         simulation.context.setPositions(self.positions)
         platform = simulation.context.getPlatform().getName()
@@ -420,10 +467,17 @@ class REM:
 
 
     def runEMNVT(self):
-        integrator = LangevinIntegrator(310*kelvin, 1/picosecond, 0.002*picoseconds)
+        integrator = LangevinIntegrator(self.T*kelvin, 1/picosecond, 0.002*picoseconds)
         system     = self.forcefield.createSystem(self.final.topology, 
                                                   nonbondedMethod=self.nonbondedMethod, 
                                                   nonbondedCutoff=self.rcut*nanometers)
+
+        # added on 20240302
+        # let protein does not change during EMNVT
+        if self.protein:
+            system.addForce(addPosrePeriodic(self.u, self.bfactor_posre, self.fcz))
+            # system.addForce(addRefPosre(self.u, self.protein, self.fcx, self.fcy, self.fcz))
+
         simulation = Simulation(self.final.topology, system, integrator)
         simulation.context.setPositions(self.positions)
         #print(simulation.context.getState().getPeriodicBoxVectors(asNumpy=True)._value * 10)
@@ -432,8 +486,10 @@ class REM:
         print("Platform: ", platform)
         print("E0: %.3e kJ/mol" %simulation.context.getState(getEnergy=True).getPotentialEnergy()._value)
         simulation.minimizeEnergy()
-        simulation.step(self.nsteps)
         print("E1: %.3e kJ/mol" %simulation.context.getState(getEnergy=True).getPotentialEnergy()._value)
+        if self.nsteps > 0:
+            simulation.step(self.nsteps)
+            print("E2: %.3e kJ/mol" %simulation.context.getState(getEnergy=True).getPotentialEnergy()._value)
         print("-------------------------------")
 
         self.positions      = simulation.context.getState(getPositions=True).getPositions()
